// --------------------------------------------------------------------
// Copyright (c) 2010 by Terasic Technologies Inc. 
// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// --------------------------------------------------------------------
//           
//                     Terasic Technologies Inc
//                     356 Fu-Shin E. Rd Sec. 1. JhuBei City,
//                     HsinChu County, Taiwan
//                     302
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// --------------------------------------------------------------------
//
// Major Functions:	XTS loopback test code
//
// --------------------------------------------------------------------
//
// Revision History :
// --------------------------------------------------------------------
//   Ver  :| Author            :| Mod. Date :| Changes Made:
//   V1.0 :| Johnny FAN        :| 01/07/10  :| Initial Revision
// --------------------------------------------------------------------

module xts_loopback_test_s4gfp (


//-----------clk-----------
	//input	[0:0]	clk_125_p,
	input	[1:1]	clk_125_p, 
	//input			clk_148_p,
	//input			clk_155_p,
//	input			clk_156_p,
	input			clkin_50,
	input			clkinbot_100_p,
	//input			clkinlt_100_p,
	//input			clkinrt_100_p,
	input			clkintop_100_p,
	output			clkout_sma,

//-----------user-----------
	input	[7:0]		user_dipsw, 
	output	[15:0]		user_led, 
	input	[2:0]		user_pb, 

//-----------cpu-----------
	input			cpu_resetn,

	//input			sma_tx_p,

//-----------hsma-----------
	input			hsma_clk_in0,
	input			hsma_clk_in_p1,
	input			hsma_clk_in_p2, 
	output			hsma_clk_out0,
	output			hsma_clk_out_p1, 
	output			hsma_clk_out_p2, 
	input	[3:0]		hsma_rx_p,  
	output [3:0]		hsma_tx_p

);


//=======================================================
//  REG/WIRE declarations
//=======================================================

wire			cal_blk_clk;				
wire	[3:0]	test_error;
wire			sys_rst_n;

//=======================================================
//  Structural coding
//=======================================================
assign sys_rst_n = user_pb[0];

cal_pll0 p0(
	.inclk0(clkintop_100_p),
	.c0(cal_blk_clk)
	);


//========== xcvr channel 0 verify ==========

xcvr_test g1(

.cal_blk_clk(cal_blk_clk),
.xcvr_refclk(clk_125_p[1]),
.irst_n(user_pb[0]),
.xcvr_tx(hsma_tx_p[0]),
.xcvr_rx(hsma_rx_p[0]),
.error_insert(user_pb[1]),
.test_error(test_error[0]),
.tx_clock(),

);

//========== xcvr channel 1 verify ==========

xcvr_test g2(

.cal_blk_clk(cal_blk_clk),
.xcvr_refclk(clk_125_p[1]),
.irst_n(user_pb[0]),
.xcvr_tx(hsma_tx_p[1]),
.xcvr_rx(hsma_rx_p[1]),
.error_insert(user_pb[1]),
.test_error(test_error[1]),
.tx_clock(),

);

//========== xcvr channel 2 verify ==========

xcvr_test g3(

.cal_blk_clk(cal_blk_clk),
.xcvr_refclk(clk_125_p[1]),
.irst_n(user_pb[0]),
.xcvr_tx(hsma_tx_p[2]),
.xcvr_rx(hsma_rx_p[2]),
.error_insert(user_pb[1]),
.test_error(test_error[2]),
.tx_clock(),

);

//========== xcvr channel 3 verify ==========
xcvr_test g4(

.cal_blk_clk(cal_blk_clk),
.xcvr_refclk(clk_125_p[1]),
.irst_n(user_pb[0]),
.xcvr_tx(hsma_tx_p[3]),
.xcvr_rx(hsma_rx_p[3]),
.error_insert(user_pb[1]),
.test_error(test_error[3]),
.tx_clock(),

);

//========== test result  ==========

assign user_led[0] =  !test_error[0];
assign user_led[1] =  !test_error[1];
assign user_led[2] =  !test_error[2];
assign user_led[3] =  !test_error[3];
///////////////////////////////////

reg	[25:0] clk_cnt;

always@(posedge clkin_50 or negedge sys_rst_n)
	begin
		if (!sys_rst_n)
			clk_cnt <= 0;
		else
			clk_cnt <= clk_cnt + 1;	
	end

assign hsma_clk_out_p1 = clk_cnt[25];
assign hsma_clk_out_p2 = clk_cnt[25];
assign hsma_clk_out0 = clk_cnt[25];

assign user_led[6] = hsma_clk_in0;
assign user_led[5] = hsma_clk_in_p1;
assign user_led[4] = hsma_clk_in_p2;

endmodule